﻿using System.Text;
using Styx.Helpers;
using System.IO;
using Styx;
using Styx.Common;
using System.ComponentModel;
using DefaultValue = Styx.Helpers.DefaultValueAttribute;
using Styx.TreeSharp;

namespace HSV
{
    public class SettingsCommon : Settings
    {
        public static readonly SettingsCommon myPrefs = new SettingsCommon();

        public SettingsCommon()
            : base(
                Path.Combine(Utilities.AssemblyDirectory,
                    string.Format(@"Routines/HSV/Settings/{0}/SettingsHSV-{1}.xml", StyxWoW.Me.RealmName, StyxWoW.Me.Name))
                )
        {
        }

        //Aoe
        [Setting]
        [DefaultValue(3)]
        [Category("Hunter")]
        [DisplayName("Aoe addcount")]
        [Description("Use Aoe Rotation if addcount >= this Value")]
        public int StartAoe { get; set; }

        public enum useMisdirect
        {
            Manual,
            Auto
        }

        [Setting]
        [DefaultValue(75)]
        [Category("Hunter")]
        [DisplayName("Mend Pet")]
        [Description("Use Mend Pet if my Pets Health <= this Value")]
        public int PercentMendPet { get; set; }

        [Setting]
        [DefaultValue(useMisdirect.Manual)]
        [Category("Hunter")]
        [DisplayName("Misdirection")]
        [Description("When to use Misdirection")]
        public useMisdirect MisdirectionUse { get; set; }

        //movement

        [Setting]
        [DefaultValue(true)]
        [Category("Movement")]
        [DisplayName("Auto Target")]
        [Description("Auto Target")]
        public bool AutoTargeting { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("Movement")]
        [DisplayName("Movement")]
        [Description("Movement")]
        public bool AutoMovement { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("Movement")]
        [DisplayName("Auto Facing")]
        [Description("Auto Facing")]
        public bool AutoFacing { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("Movement in Raids, Instances, Dungeons")]
        [DisplayName("Disable Auto Move")]
        [Description("Disable Auto Movement in Raids, Instances, Dungeons")]
        public bool AutoMovementRaid { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("Movement in Raids, Instances, Dungeons")]
        [DisplayName("Disable Auto Target")]
        [Description("Disable Auto Targeting in Raids, Instances, Dungeons")]
        public bool AutoTargetingRaid { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("Movement in Raids, Instances, Dungeons")]
        [DisplayName("Disable Auto Facing")]
        [Description("Disable Auto Facing in Raids, Instances, Dungeons")]
        public bool AutoFacingRaid { get; set; }

        //flasks
        public enum FlaskUse
        {
            Manual,
            Raids,
            Raids_but_not_LFR,
            Dungeons
        }
        [Setting]
        [DefaultValue(false)]
        [Category("Flasks & Potions")]
        [DisplayName("Alchemy Flask")]
        [Description("Auto use Alchemy Flask when no more powerfull buff exists")]
        public bool AutoUseAlchemyFlask { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("Flasks & Potions")]
        [DisplayName("Crystal of Insanity")]
        [Description("Auto use Crystal of Insanity when no FlaskBuff exists")]
        public bool AutoUseCrystalOfInsanity { get; set; }

        [Setting]
        [DefaultValue(FlaskUse.Manual)]
        [Category("Flasks & Potions")]
        [DisplayName("Flask of Spring Blossom")]
        [Description("Auto use Flask of Spring Blossom")]
        public FlaskUse AutoUseFlaskOfSpringBlossom { get; set; }

        // trinkets section
        public enum TrinketsUse
        {
            Manual,
            Below_HealthPercent,
            CrowdControlled,
            DPS_Boost

        }

        [Setting]
        [DefaultValue(TrinketsUse.Manual)]
        [Category("Trinket 1")]
        [DisplayName("Trinket 1")]
        [Description("When to use trinket 1")]
        public TrinketsUse Trinket1Use { get; set; }

        [Setting]
        [DefaultValue(65)]
        [Category("Trinket 1")]
        [DisplayName("Trinket 1 HP %")]
        [Description("Use trinket 1 below HP %")]
        public int trinket1hppercent { get; set; }

        [Setting]
        [DefaultValue(TrinketsUse.Manual)]
        [Category("Trinket 2")]
        [DisplayName("Trinket 2")]
        [Description("When to use trinket 2")]
        public TrinketsUse Trinket2Use { get; set; }

        [Setting]
        [DefaultValue(65)]
        [Category("Trinket 2")]
        [DisplayName("Trinket 2 HP %")]
        [Description("Use trinket 2 below HP %")]
        public int trinket2hppercent { get; set; }

        public enum useEngiGloves
        {
            Manual,
            OnCoolDown,
            BossFights
        }

        [Setting]
        [DefaultValue(useEngiGloves.Manual)]
        [Category("Engineer Gloves")]
        [DisplayName("Use Engineer Gloves")]
        [Description("Use Engineer Gloves")]
        public useEngiGloves GlovesChoice { get; set; }

        //resting

        [Setting]
        [DefaultValue(45)]
        [Category("Resting")]
        [DisplayName("Eat Food HP %")]
        [Description("Eat Food below HP % Out of Combat")]
        public int PercentEatFood { get; set; }

        [Setting]
        [DefaultValue(55)]
        [Category("Hunter")]
        [DisplayName("Healthstone HP %")]
        [Description("Use Healthstone below HP %")]
        public int PercentHealthstone { get; set; }

        [Setting]
        [DefaultValue(0)]
        [Category("Hunter")]
        [DisplayName("Disengage distance")]
        [Description("Use Disengage when target closer than Value. 0 to disable")]
        public int DisengageDistance { get; set; }


        [Setting]
        [DefaultValue(1)]
        [Category("Pet")]
        [DisplayName("Call Solo Pet")]
        [Description("Call Solo Pet")]
        public int CallSoloPet { get; set; }

        [Setting]
        [DefaultValue(2)]
        [Category("Pet")]
        [DisplayName("Call Raid Pet")]
        [Description("Call Raid Pet")]
        public int CallRaidPet { get; set; }

        [Setting]
        [DefaultValue(2)]
        [Category("Pet")]
        [DisplayName("Call BG Pet")]
        [Description("Call BG Pet")]
        public int CallBgPet { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("Pet")]
        [DisplayName("Auto Call Pet")]
        [Description("Auto Call Pet")]
        public bool CallAutoPet { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("Hunter")]
        [DisplayName("Auto Interrupt")]
        [Description("Auto Interrupt")]
        public bool autoInterrupt { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("Timeless Isle")]
        [DisplayName("Auto Buffs")]
        [Description("Auto Buff Timeless Isle Buffs")]
        public bool AutoTimelessIsleBuffs { get; set; }


    }
}
